"""
standard - standard star tools
2007.08.23. YOSHIKAWA, Tomohiro
$Log: not supported by cvs2svn $
Revision 1.2  2007/08/24 08:19:59  tomohiro
modify to use ADU/A/sec instead of ADU/sec

Revision 1.1  2007/08/24 07:19:37  tomohiro
standard star tools

"""
import numpy,pyfits
from iraftool import *
from spectool import *
from astrocat import gaia

def rescurve(stddata, stdmag, output, 
             magfilter="mdpdb$filter/2mass_K.fits",
             moddata="mdpdb$standard/A0V_g50.spc",
             vegamodel="mdpdb$filter/alpha_lyr_stis_003.fits"):
    """
    stddata:   one-D standard star data
    stdmag:    Vega magnitude of the standard at magfilter
    output:    output response curve
    magfilter: filter curve for stdmag
    moddata:  standard star model spectrum
    vegamedel: Vega model spectrum
    """
    # load Vega model
    vega = pyfits.getdata(iraf.osfn(vegamodel))
    vega_nir_index = (8000 < vega.field('WAVELENGTH'))&(vega.field('WAVELENGTH') < 25000)
    vega_nir = vega[vega_nir_index]
    # load standard star model
    mod = gaia.open(iraf.osfn(moddata)).getrecord()
    flt = pyfits.getdata(iraf.osfn(magfilter))
    # load standard star data
    std = specdata.load_onedspec(iraf.osfn(stddata))
    
    Fvega  = specdata.bandpass(vega_nir, flt, fnu=True, photon=True)
    Fmodel = specdata.bandpass(mod, flt, fnu=True, photon=True)
    
    # scale stdmodel
    mod_sc = specdata.spec_arith(mod, '*', Fvega/Fmodel*10**(-stdmag/2.5))
    
    # devide stddata by scaled model
    std_r = specdata.spec_arith(std, '/', mod_sc)

    # normalized by exposure time and wavelength
    stdhdr  = pyfits.getheader(iraf.osfn(stddata))
    exptime = stdhdr['EXPTIME']
    cdelt1  = stdhdr['CDELT1']
    std_rn  = specdata.spec_arith(std_r, '/', exptime)
    std_rna = specdata.spec_arith(std_rn, '/', cdelt1)
    
    pyfits.BinTableHDU(std_rna, header=stdhdr).writeto(fileutil.osimg(output))

def mdpfcalib(input, rescurve, result):
    """
    perform flux calibration to input frames using rescurve
    """
    listi = fileutil.fileparse(input, check=True, ext=True)
    listr = fileutil.fileparse(result, ext=True)
    
    fc = pyfits.open(fileutil.osimg(rescurve))
    for filei, filer in zip(listi, listr):
        fi = pyfits.open(fileutil.osimg(filei[0]))
        
        # get header data
        exptime = fi[0].header['EXPTIME']
        cdelt = fi[0].header['CDELT1']

        ## object frame
        # cast data type to float64
        fi[0].scale('float64')
        
        # normalize by exposure time
        fi[0] = specdata.twodspec_arith(fi[0], '/', exptime)
        fi[0].header['EXPTIME'] = 1
        
        # flux calibration
        specdata.twodspec_arith(fi[0], '/', cdelt)
        specdata.twodspec_arith(fi[0], '/', fc[1].data)
        
        if len(fi)>1:
            # cast data type to float64
            fi[1].scale('float64')
            
            fi[1] = specdata.twodspec_arith(fi[1], '/', exptime)
            fi[1].header['EXPTIME'] = 1
            
            # noise 
            sigd = numpy.sqrt(fi[1].data/fi[0].header['GAIN'])
            sig = pyfits.ImageHDU(data=sigd, header=fi[0].header)
            specdata.twodspec_arith(sig, '/', cdelt)
            specdata.twodspec_arith(sig, '/', fc[1].data)

            # background frame
            specdata.twodspec_arith(fi[1], '/', cdelt)
            specdata.twodspec_arith(fi[1], '/', fc[1].data)
        
            fi.append(sig)
        
        fi[0].header.update('MDP_RESP', rescurve, comment='response curve file name')
        fi.writeto(fileutil.osimg(filer[0]))

def onedspec2specdata(infile, outfile):
    """
    convert onedspec file to binary table FITS file
    """
    for input, output in zip(fileutil.fileparse(infile), fileutil.fileparse(outfile)):
        print input + '->' + output
        specdata.load_onedspec(fileutil.osimg(input), outfile=fileutil.osimg(output), pera=True)

def bandpass(infile, filter, fnu=True, photon=True):
    for input in fileutil.fileparse(infile):
        inpf = pyfits.open(fileutil.osimg(input))
        flpf = pyfits.open(fileutil.osimg(filter))
        exptime = inpf[1].header['EXPTIME']
        print specdata.bandpass(inpf[1].data, flpf[1].data, fnu=fnu, photon=photon)/exptime

