# ds9.py
# 2006.06.20 YOSHIKAWA, Tomohiro

from RO import DS9
from mcsmdp import shape
from mcsmdp import object
from mcsmdp import instrument
from astrocat import datalist
import __builtin__
from types import *

class Ds9(DS9.DS9Win):
    """class ds9
    """
    def __init__(self):
        DS9.DS9Win.__init__(self)
        self.objlist = {}
        self._regmax = 500  # maximum number of region
        self.ins = None
    
    def set_instrument(self, instfile):
        self.inst = instrument.LoadFile(instfile)
        
    def display(self, file, frame=1,
                scale='linear', mode='zscale', z1=None, z2=None,
                cmap='grey', invert=False, zoom='fit'):
        """method display
        display FITS file on ds9
        file: FITS file name
        frame: frame number to display
        scale: scale mode (linear|log|pow|sqrt|squared|histequ)
        mode: auto scale mode (minmax|<value>|zscale|zmax)
        z1, z2: manual scale
        cmap: color map
        invert: invert color map
        zoom: fit or numeric value
        """
        self.xpaset('frame %d'%(frame))
        self.showFITSFile(file)
        self.xpaset('scale '+scale)
        if z1 != None and z2 != None:
            self.xpaset('scale limits %f %f'%(z1,z2))
        else:
            self.xpaset("scale mode %s"%(str(mode)))
        self.xpaset('cmap '+cmap)
        if invert:
            self.xpaset('cmap invert yes')
        self.xpaset('zoom to '+str(zoom))
        line = self.xpaget('scale limits')
        line = line.strip()
        z1, z2 = line.split()[:2]
        print "z1=%s z2=%s"%(z1,z2)

    def mdisplay(self, filelist, frame=1,
                scale='linear', mode='zscale', z1=None, z2=None,
                cmap='BB', invert=False):
        """method display
        display FITS file on ds9
        filelist: list of FITS file name
        frame: frame number to display
        scale: scale mode (linear|log|pow|sqrt|squared|histequ)
        mode: auto scale mode (minmax|<value>|zscale|zmax)
        z1, z2: manual scale
        """
        for file in filelist:
            self.display(file.strip(), frame=frame, scale=scale, mode=mode, z1=z1, z2=z2,
                         cmap=cmap, invert=invert)
            frame += 1

    def clreg(self):
        """
        clear all regions
        """
        self.xpaset('regions delete all')

    def readcur(self, coord='image', coord_unit=''):
        """
        read cursor clicked position
        coord: image|wcs
        coord_unit: degree|sexagesimal for coord=wcs
        """
        if coord == 'wcs':
            if coord_unit == 'sexagesimal':
                xpastr = 'wcs fk5 sexagesimal'
            else:
                xpastr = 'wcs fk5 degrees'
        else:
            xpastr = 'image'
        line = self.xpaget('imexam coordinate '+xpastr)
        line = line.strip()
        data = line.split()[:2]
        return float(data[0]), float(data[1])

    def plotlist(self, slist, move=0, rotate=0, edit=0):
        regtmp = ''
        regnum = self._regmax
        
        for sp in slist.list:
            if not sp.enable: continue
            regtmp = regtmp + sp.getreg(move=move, rotate=rotate, edit=edit)+';'
            regnum -= 1
            if regnum < 1:
                #print regtmp
                self.xpaset('regions', data=regtmp)
                regtmp=''
                regnum = self._regmax
        if len(regtmp) > 0:
            #print regtmp
            self.xpaset('regions', data=regtmp)
    
    ###############
    # object list #
    ###############
    def getinfo(self, coord='fk5'):
        """method getinfo
        get information about selected object
        """
        self.xpaset('regions system '+coord)
        self.xpaset('regions skyformat degrees')
        regstr = self.xpaget('regions selected')
        splist = []
        for line in regstr.split('\n'):
            if len(line) == 0: continue
            if line[0] == '#': continue
            if line[:6] == 'global': continue
            if line[:len(coord)] == coord: continue
            sp = shape.reg2shape(line, coord)
            splist.append(sp)
        return splist

    def printinfo(self, objlist=None):
        """method printinfo
        print information about selected objects
        """
        splist = self.getinfo()
        printlist = []
        for index in range(len(splist)):
            id = splist[index].id
            if objlist != None:
                obj = objlist.getbyid(id)
            elif len(self.objlist) <= 0:
                return
            else:
                for key in self.objlist.keys():
                    obj = self.objlist[key].getbyid(id)
                    if obj != None: break
                if obj == None: return
            ## print header
            if index == 0:
                if obj.coord == 'fk5': hdstr = "RA\tDEC\t"
                else: hdstr = "X\tY\t"
                for k,v in obj.data.iteritems():
                    hdstr = hdstr + k + "\t"
                print hdstr
                print "--------------------------"
            
            print str(obj.x)+"\t"+str(obj.y)+"\t",
            for k,v in obj.data.iteritems():
                print str(v)+"\t",
            print

    def printdata(self, keys=None):
        """method print data
        print data about selected object
        """
        splist = self.getinfo()
        if len(splist) <= 0: return
        id = splist[0].id
        if len(self.objlist) <= 0:
            return
        else:
            for key in self.objlist.keys():
                obj = self.objlist[key].getbyid(id)
                if obj != None: break
            if obj == None: return
        if keys == None:
            for k,v in obj.data.iteritems():
                print "%s\t\t=\t%s"%(k,v)
        else:
            for k in keys:
                print "%s\t\t=\t%s"%(k,obj.data[k])

    def addcat(self, file, ftype='gaia', mark='circle', color='green', size=None, size_unit='arcsec',
                label=None, coord='fk5', coord_unit='degree', group=None, xname=None, yname=None):
        """method addcat
        add catalog file to ds9 object
        """
        objlist = object.LoadList(file, type=ftype, coord=coord, coord_unit=coord_unit, group=group, xname=xname, yname=yname)
        objlist.setprop(mark=mark, size=size, size_unit=size_unit, color=color, move=0, rotate=0, edit=0)
        objlist.setlabel(label)
        self.objlist[objlist.groupid]=objlist
        return objlist
    def adddata(self, data, mark='circle', color='green', size=None, size_unit='arcsec',
                label=None, coord='fk5', coord_unit='degree', group=None, xname=None, yname=None):
        """method datacat
        add catalog datalist to ds9 object
        """
        objlist = object.ObjectList()
        objlist.loaddata(data, coord=coord, coord_unit=coord_unit, group=group, xname=xname, yname=yname)
        objlist.setprop(mark=mark, size=size, size_unit=size_unit, color=color, move=0, rotate=0, edit=0)
        objlist.setlabel(label)
        self.objlist[objlist.groupid]=objlist
        return objlist

    def plotobj(self, name, mark=None, color=None, size=None, size_unit=None, move=0, rotate=0, edit=0):
        slist = self.objlist[name]
        slist.setprop(mark=mark, size=size, size_unit=size_unit, color=color, move=0, rotate=0, edit=0)

        self.plotlist(slist, move=move, rotate=rotate, edit=edit)

    def hideobj(self, name):
        slist = self.objlist[name]
        if self.groupid != None:
            self.xpaset('regions group gid:%s delete'%(slist.groupid))
        else:
            for sp in slist:
                self.xpaset('regouns group id:%s delete'%(sp.id))
    
    def printlist(self):
        """method printlist
        print catalog list
        """
        print "---------------------------------"
        print "current catalog list:"
        for k, v in self.objlist.iteritems():
            print "%s:\t%d objects"%(k,len(v.list))
        print "---------------------------------"

    def search(self, key, value, action='select'):
        self.xpaset('regions select none')
        if len(self.objlist) <= 0:
            return
        for cat in self.objlist.itervalues():
            if type(value) == StringType:
                value = [value]
            #print key, value
            for v in value:
                objlist = cat.search(key,datalist.str2data(v))
                for obj in objlist:
                    self.xpaset('regions group id:%s %s'%(obj.id,action))

    #################
    # FOV           #
    #################
    def showfov(self, name=False, slitarea=True):
        self.xpaset('regions', data=self.inst.getreg(name=name, slitarea=slitarea))
    def hidefov(self):
        self.xpaset('regions group id:%s delete'%self.inst.fov.id)
        self.xpaset('regions group id:%s delete'%self.inst.slit_area.id)

    #################
    # SLIT          #
    #################
    def showslit(self, slitdata):
        self.plotlist(slitdata)
    def hideslit(self):
        self.xpaset('regions group gid:slit delete')
        self.xpaset('regions group gid:hole delete')
        
    def showspec(self, slitdata):
        self.plotlist(slitdata.speclist)
    def hidespec(self):
        self.xpaset('regions group gid:spec delete')

    #######################
    # TVMARK              #
    #######################
    def plotmark(self, x, y, coord='image', coord_unit='default',
                 mark='circle', size=None, size_unit='default', color='yellow',
                 label=None, **kargs):
        if (label != None):
            label = "{%s}"%(label)
            obj = object.Object(x,y,coord,coord_unit,mark,size,size_unit,color,text=label)
        else:
            obj = object.Object(x,y,coord,coord_unit,mark,size,size_unit,color)
        self.xpaset('regions', data=obj.getreg())
    
    def tvmark(self, file, frame=None,
               coord='image', coord_unit='default',
               mark='circle', size=None, size_unit='default', color='yellow',
               label=False, number=False):
        if frame != None:
            self.xpaset('frame %d'%frame)
        i = 1
        for line in __builtin__.open(file):
            if line[0] == '#' or line[0] == '\n': continue
            line = line.strip()
            if (label):
                x,y,lb = line.split()[:3]
            else:
                x,y = line.split()[:2]
                if (number):
                    lb = str(i)
                    i = i+1
            if (label or number):
                self.plotmark(x,y,coord,coord_unit,mark,size,size_unit,color,label=lb)
            else:
                #print x,y
                self.plotmark(x,y,coord,coord_unit,mark,size,size_unit,color)
    
    #######################
    # OBSOLETE            #
    #######################
    def plotcat(self, file, ftype='gaia', mark='circle', color='green', size=None, size_unit='arcsec',
                label=None, coord='fk5', coord_unit='degree', group=None, xname=None, yname=None):
        """method plotcat
        plot catalog to ds9
        """
        objlist = self.addcat(file, ftype=ftype, coord=coord, coord_unit=coord_unit, group=group, xname=xname, yname=yname,
                              mark=mark, size=size, size_unit=size_unit, color=color)
        objlist.show()


def open():
    """function open()
    constructor for ds9
    """
    return Ds9()
